@@ -15,8 +15,6 @@
  */
 
 #include <fcntl.h>
-#include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include "afile.h"
@@ -536,7 +534,7 @@ afile_aiff_readhdr(struct afile *f)
 	unsigned int csize, rsize, nfr = 0, pos = 0, offs;
 	int comm_done = 0, comp;
 
-	if (!afile_readhdr(f, &form, sizeof(struct wav_riff)))
+	if (!afile_readhdr(f, &form, sizeof(struct aiff_form)))
 		return 0;
 	if (memcmp(&form.id, &aiff_id_form, 4) != 0) {
 		log_puts(f->path);
@@ -563,7 +561,7 @@ afile_aiff_readhdr(struct afile *f)
 			log_puts(f->path);
 			log_puts(": failed to read chunk header\n");
 			return 0;
-		}		
+		}
 		csize = be32_get(&chunk.size);
 		if (memcmp(chunk.id, aiff_id_comm, 4) == 0) {
 			if (!afile_aiff_readcomm(f, csize, comp, &nfr))
@@ -572,7 +570,8 @@ afile_aiff_readhdr(struct afile *f)
 		} else if (memcmp(chunk.id, aiff_id_data, 4) == 0) {
 			if (!afile_aiff_readdata(f, csize, &offs))
 				return 0;
-			f->startpos = sizeof(form) + pos + sizeof(chunk) + offs;
+			f->startpos = sizeof(form) + pos +
+			    sizeof(chunk) + offs;
 			break;
 		} else {
 #ifdef DEBUG
@@ -657,7 +656,7 @@ afile_au_readhdr(struct afile *f)
 	struct au_hdr hdr;
 	unsigned int fmt;
 
-	if (!afile_readhdr(f, &hdr, sizeof(struct wav_riff)))
+	if (!afile_readhdr(f, &hdr, sizeof(struct au_hdr)))
 		return 0;
 	if (memcmp(&hdr.id, &au_id, 4) != 0) {
 		log_puts(f->path);
@@ -687,17 +686,14 @@ afile_au_readhdr(struct afile *f)
 	case AU_FMT_ULAW:
 		f->fmt = AFILE_FMT_ULAW;
 		f->par.bits = 8;
-		f->par.bps = 1;
 		break;
 	case AU_FMT_ALAW:
 		f->fmt = AFILE_FMT_ALAW;
 		f->par.bits = 8;
-		f->par.bps = 1;
 		break;
 	case AU_FMT_FLOAT:
 		f->fmt = AFILE_FMT_FLOAT;
 		f->par.bits = 32;
-		f->par.bps = 4;
 		break;
 	default:
 		log_puts(f->path);
@@ -927,7 +923,8 @@ afile_open(struct afile *f, char *path, int hdr, int flags,
 			f->fd = STDOUT_FILENO;
 		} else {
 			f->path = path;
-			f->fd = open(f->path, O_WRONLY | O_TRUNC | O_CREAT, 0666);
+			f->fd = open(f->path,
+			    O_WRONLY | O_TRUNC | O_CREAT, 0666);
 			if (f->fd < 0) {
 				log_puts(f->path);
 				log_puts(": failed to create file\n");
@@ -956,7 +953,8 @@ afile_open(struct afile *f, char *path, int hdr, int flags,
 				f->par.msb = 1;
 			f->endpos = f->startpos = sizeof(struct aiff_hdr);
 			f->maxpos = 0x7fffffff;
-			if (!afile_writehdr(f, &dummy, sizeof(struct aiff_hdr)))
+			if (!afile_writehdr(f, &dummy,
+				sizeof(struct aiff_hdr)))
 				goto bad_close;
 		} else if (f->hdr == AFILE_HDR_AU) {
 			f->par.bits = (f->par.bits + 7) & ~7;
