@@ -40,7 +40,7 @@
 #ifndef HAVE_ARC4RANDOM
 
 #ifndef DEV_RANDOM
-#define DEV_RANDOM "/dev/arandom"
+#define DEV_RANDOM "/dev/urandom"
 #endif
 
 static int
@@ -335,10 +335,8 @@ bad_gen:
 		unlink(tmp);
 	}
 done:
-	if (tmp)
-		free(tmp);
-	if (path)
-		free(path);
+	free(tmp);
+	free(path);
 	return 1;
 }
 
@@ -366,6 +364,14 @@ aucat_connect_tcp(struct aucat *hdl, char *host, unsigned int unit)
 			DPERROR("socket");
 			continue;
 		}
+#ifndef HAVE_SOCK_CLOEXEC
+		if (fcntl(s, F_SETFL, FD_CLOEXEC) < 0) {
+			DPERROR("FD_CLOEXEC");
+			close(s);
+			s = - 1;
+			continue;
+		}
+#endif
 	restart:
 		if (connect(s, ai->ai_addr, ai->ai_addrlen) < 0) {
 			if (errno == EINTR)
@@ -405,6 +411,13 @@ aucat_connect_un(struct aucat *hdl, unsigned int unit)
 	s = socket(AF_UNIX, SOCK_STREAM | SOCK_CLOEXEC, 0);
 	if (s < 0)
 		return 0;
+#ifndef HAVE_SOCK_CLOEXEC
+	if (fcntl(s, F_SETFL, FD_CLOEXEC) < 0) {
+		DPERROR("FD_CLOEXEC");
+		close(s);
+		return 0;
+	}
+#endif
 	while (connect(s, (struct sockaddr *)&ca, len) < 0) {
 		if (errno == EINTR)
 			continue;
