@@ -28,7 +28,7 @@
 #include "utils.h"
 
 void zomb_onmove(void *);
-void zomb_onvol(void *, unsigned int);
+void zomb_onvol(void *);
 void zomb_fill(void *);
 void zomb_flush(void *);
 void zomb_eof(void *);
@@ -60,7 +60,7 @@ void dev_cycle(struct dev *);
 int dev_getpos(struct dev *);
 struct dev *dev_new(char *, struct aparams *, unsigned int, unsigned int,
     unsigned int, unsigned int, unsigned int, unsigned int);
-void dev_adjpar(struct dev *, int, int, int, int, int);
+void dev_adjpar(struct dev *, int, int, int);
 int dev_open(struct dev *);
 void dev_close(struct dev *);
 int dev_ref(struct dev *);
@@ -160,7 +160,7 @@ zomb_onmove(void *arg)
 }
 
 void
-zomb_onvol(void *arg, unsigned int vol)
+zomb_onvol(void *arg)
 {
 }
 
@@ -1022,7 +1022,7 @@ dev_new(char *path, struct aparams *par,
  */
 void
 dev_adjpar(struct dev *d, int mode,
-    int pmin, int pmax, int rmin, int rmax)
+    int pmax, int rmax)
 {
 	d->reqmode |= mode & MODE_AUDIOMASK;
 	if (mode & MODE_PLAY) {
@@ -1427,7 +1427,7 @@ slot_new(struct dev *d, char *who, struct slotops *ops, void *arg, int mode)
 	struct slot *s;
 
 	/*
-	 * create a ``valid'' control name (lowcase, remove [^a-z], trucate)
+	 * create a ``valid'' control name (lowcase, remove [^a-z], truncate)
 	 */
 	for (i = 0, p = who; ; p++) {
 		if (i == SLOT_NAMEMAX - 1 || *p == '\0') {
@@ -1515,19 +1515,19 @@ slot_new(struct dev *d, char *who, struct slotops *ops, void *arg, int mode)
 found:
 	if (!dev_ref(d))
 		return NULL;
+	if ((mode & d->mode) != mode) {
+		if (log_level >= 1) {
+			slot_log(s);
+			log_puts(": requested mode not supported\n");
+		}
+		dev_unref(d);
+		return 0;
+	}
 	s->dev = d;
 	s->ops = ops;
 	s->arg = arg;
 	s->pstate = SLOT_INIT;
 	s->tstate = MMC_OFF;
-
-	if ((mode & s->dev->mode) != mode) {
-		if (log_level >= 1) {
-			slot_log(s);
-			log_puts(": requested mode not supported\n");
-		}
-		return 0;
-	}
 	s->mode = mode;
 	aparams_init(&s->par);
 	if (s->mode & MODE_PLAY) {
@@ -1712,6 +1712,23 @@ slot_attach(struct slot *s)
 			    xmalloc(s->round * slot_nch * sizeof(adata_t));
 		}
 
+		/*
+		 * cmap_copy() doesn't write samples in all channels,
+	         * for instance when mono->stereo conversion is
+	         * disabled. So we have to prefill cmap_copy() output
+	         * with silence.
+	         */
+		if (s->sub.resampbuf) {
+			memset(s->sub.resampbuf, 0,
+			    d->round * slot_nch * sizeof(adata_t));
+		} else if (s->sub.encbuf) {
+			memset(s->sub.encbuf, 0,
+			    s->round * slot_nch * sizeof(adata_t));
+		} else {
+			memset(s->sub.buf.data, 0,
+			    s->appbufsz * slot_nch * sizeof(adata_t));
+		}
+
 		/*
 		 * N-th recorded block is the N-th played block
 		 */
@@ -1824,7 +1841,7 @@ slot_detach(struct slot *s)
 #endif
 	for (ps = &s->dev->slot_list; *ps != s; ps = &(*ps)->next) {
 #ifdef DEBUG
-		if (s == NULL) {
+		if (*ps == NULL) {
 			slot_log(s);
 			log_puts(": can't detach, not on list\n");
 			panic();
