@@ -304,8 +304,6 @@ resamp_init(struct resamp *p, unsigned int iblksz,
 	p->iblksz = iblksz;
 	p->oblksz = oblksz;
 	p->diff = 0;
-	p->idelta = 0;
-	p->odelta = 0;
 	p->nch = nch;
 	p->ctx_start = 0;
 	for (i = 0; i < NCHAN_MAX * RESAMP_NCTX; i++)
@@ -469,7 +467,7 @@ enc_init(struct conv *p, struct aparams *par, int nch)
 }
 
 /*
- * decode "todo" frames from from foreign to native encoding
+ * decode "todo" frames from foreign to native encoding
  */
 void
 dec_do(struct conv *p, unsigned char *in, unsigned char *out, int todo)
@@ -608,7 +606,7 @@ cmap_add(struct cmap *p, void *in, void *out, int vol, int todo)
 }
 
 /*
- * overwrite output with "todo" input frames with with the given volume
+ * overwrite output with "todo" input frames with the given volume
  */
 void
 cmap_copy(struct cmap *p, void *in, void *out, int vol, int todo)
