@@ -39,7 +39,7 @@ void sock_slot_fill(void *);
 void sock_slot_flush(void *);
 void sock_slot_eof(void *);
 void sock_slot_onmove(void *);
-void sock_slot_onvol(void *, unsigned int);
+void sock_slot_onvol(void *);
 void sock_midi_imsg(void *, unsigned char *, int);
 void sock_midi_omsg(void *, unsigned char *, int);
 void sock_midi_fill(void *, int);
@@ -227,7 +227,7 @@ sock_slot_onmove(void *arg)
 }
 
 void
-sock_slot_onvol(void *arg, unsigned int delta)
+sock_slot_onvol(void *arg)
 {
 	struct sock *f = (struct sock *)arg;
 	struct slot *s = f->slot;
@@ -314,7 +314,7 @@ sock_exit(void *arg)
 }
 
 /*
- * write on the socke fd and handle errors
+ * write on the socket fd and handle errors
  */
 int
 sock_fdwrite(struct sock *f, void *data, int count)
@@ -355,7 +355,7 @@ sock_fdwrite(struct sock *f, void *data, int count)
 }
 
 /*
- * read from the socke fd and handle errors
+ * read from the socket fd and handle errors
  */
 int
 sock_fdread(struct sock *f, void *data, int count)
@@ -509,7 +509,7 @@ sock_rdata(struct sock *f)
 }
 
 /*
- * read data into the slot/midi ring buffer
+ * write data to the slot/midi ring buffer
  */
 int
 sock_wdata(struct sock *f)
@@ -541,6 +541,10 @@ sock_wdata(struct sock *f)
 		return 1;
 	}
 	while (f->wtodo > 0) {
+		/*
+		 * f->slot and f->midi are set by sock_hello(), so
+		 * count is always properly initialized
+		 */
 		if (f->slot)
 			data = abuf_rgetblk(&f->slot->sub.buf, &count);
 		else if (f->midi)
@@ -622,7 +626,7 @@ sock_setpar(struct sock *f)
 	if (AMSG_ISSET(rchan) && (s->mode & MODE_RECMASK)) {
 		if (rchan < 1)
 			rchan = 1;
-		if (rchan > NCHAN_MAX)
+		else if (rchan > NCHAN_MAX)
 			rchan = NCHAN_MAX;
 		s->sub.slot_cmin = f->opt->rmin;
 		s->sub.slot_cmax = f->opt->rmin + rchan - 1;
@@ -646,7 +650,7 @@ sock_setpar(struct sock *f)
 	if (AMSG_ISSET(pchan) && (s->mode & MODE_PLAY)) {
 		if (pchan < 1)
 			pchan = 1;
-		if (pchan > NCHAN_MAX)
+		else if (pchan > NCHAN_MAX)
 			pchan = NCHAN_MAX;
 		s->mix.slot_cmin = f->opt->pmin;
 		s->mix.slot_cmax = f->opt->pmin + pchan - 1;
@@ -670,7 +674,7 @@ sock_setpar(struct sock *f)
 	if (AMSG_ISSET(rate)) {
 		if (rate < RATE_MIN)
 			rate = RATE_MIN;
-		if (rate > RATE_MAX)
+		else if (rate > RATE_MAX)
 			rate = RATE_MAX;
 		s->round = dev_roundof(d, rate);
 		s->rate = rate;
@@ -1509,6 +1513,11 @@ sock_write(struct sock *f)
 	case SOCK_WMSG:
 		if (!sock_wmsg(f))
 			return 0;
+		/*
+		 * f->wmsg is either build by sock_buildmsg() or
+		 * copied from f->rmsg (in the SOCK_RRET state), so
+		 * it's safe.
+		 */
 		if (ntohl(f->wmsg.cmd) != AMSG_DATA) {
 			f->wstate = SOCK_WIDLE;
 			f->wtodo = 0xdeadbeef;
